@@ -1,5 +1,7 @@
 from __future__ import absolute_import
 
+import logging
+
 from kombu import Connection
 from kombu.common import itermessages, maybe_declare
 
@@ -7,6 +9,9 @@ from nameko.containers import WorkerContext
 from nameko.rpc import ServiceProxy, ReplyListener
 
 
+_logger = logging.getLogger(__name__)
+
+
 class ConsumeEvent(object):
     """ Event for the RPC consumer with the same interface as eventlet.Event.
     """
@@ -27,7 +32,7 @@ class ConsumeEvent(object):
 
         Exceptions are raised directly.
         """
-        self.queue_consumer.poll_messages(self.correlation_id)
+        self.queue_consumer.get_message(self.correlation_id)
         return self.body
 
 
@@ -42,7 +47,11 @@ class PollingQueueConsumer(object):
         self.connection = Connection(provider.container.config['AMQP_URI'])
         self.channel = self.connection.channel()
         self.queue = provider.queue
+        self.queue.auto_delete = True
         maybe_declare(self.queue, self.channel)
+        message_iterator = self._poll_messages()
+        message_iterator.send(None)  # start generator
+        self.get_message = message_iterator.send
 
     def unregister_provider(self, provider):
         self.connection.close()
@@ -50,14 +59,18 @@ class PollingQueueConsumer(object):
     def ack_message(self, msg):
         msg.ack()
 
-    def poll_messages(self, correlation_id):
+    def _poll_messages(self):
         channel = self.channel
         conn = channel.connection
 
+        correlation_id = yield
+
         for body, msg in itermessages(conn, channel, self.queue, limit=None):
-            if correlation_id == msg.properties.get('correlation_id'):
-                self.provider.handle_message(body, msg)
-                break
+            msg_correlation_id = msg.properties.get('correlation_id')
+            if msg_correlation_id == correlation_id:
+                correlation_id = yield self.provider.handle_message(body, msg)
+            else:
+                _logger.debug("Unknown correlation id: %s", msg_correlation_id)
 
 
 class SingleThreadedReplyListener(ReplyListener):
